/* $Id: visual.c,v 1.2 2005/12/20 22:27:20 cegger Exp $
***************************************************************************

   X server for LibGGI - visual implementation

   Copyright (C) 1997	   Michael Krause	[rawstyle@ms.demo.org]
   Copyright (C) 1998-1999 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

***************************************************************************
*/

#define PSZ 8
#include <ggi/gg.h>

#include "mi.h"
#include "cfb.h"
#include "cfb16.h"
#include "cfb24.h"
#include "cfb32.h"

#include "scrnintstr.h"
#include "mibstore.h"
#include "gcstruct.h"

#include "xggi.h"
#include "xggi_misc.h"



static void reset_xggivisual(struct xggiVisualInfo *visual)
{
	memset(visual, 0, sizeof(struct xggiVisualInfo));

	visual->vis = NULL;
	visual->width = GGI_AUTO;
	visual->height = GGI_AUTO;
	visual->depth = 0;
	visual->stride = 0;
	visual->bitsPerPixel = 0;
	/* visual->mode = */
	visual->dbuf = NULL;
	visual->pfbMemory = NULL;

	return;
}


int xggiVisualAlloc(struct xggiScreenInfo *screen, int numvisuals)
{
	int i;

	screen->visuals = safe_alloc(sizeof(struct xggiVisualInfo)
					* numvisuals);
	screen->numvisuals = numvisuals;

	for (i = 0; i < screen->numvisuals; i++) {
		reset_xggivisual(&(screen->visuals[i]));
	}

	return 0;
}


int xggiVisualFree(struct xggiScreenInfo *screen)
{
	int i;

	for (i = 0; i < screen->numvisuals; i++) {
		if (screen->visuals[i].vis != NULL) {
			ggiClose(screen->visuals[i].vis);
		}
	}

	xfree(screen->visuals);
	screen->visuals = NULL;
	screen->numvisuals = 0;

	return 0;
}



int xggiGetVisualIdx(int scr, VisualID visualid)
{
	int i;

	if (scr < 0 || scr > xggiInfo.numscreens) {
		FatalError("xggiGetVisualIdx: Tried to operate on invalid screen\n");
	}

	for (i = 0; i < xggiInfo.numscreens; i++) {
		if (VISUAL_ID(scr, i) == visualid) return i;
	}

#if 0	/* What's this? */
	*((uint32_t*)0) = 43;
#endif
	FatalError("xggiGetVisualIdx: Tried to operate on invalid visual\n");

	/* Never reached */
	return -1;
}

